#include <DatFile.h>
#include <DatSubsystems/DatExporter.h>
#include <DatSubsystems/DatFileSystem.h>
#include <BinaryData.h>
#include <SubfileData.h>
#include <SubFile.h>

#include <string>
#include <fstream>
#include <algorithm>

namespace LOTRO_DAT {
    DatExporter::DatExporter(DatFile *datFilePtr) : dat(datFilePtr) {
    }

    DatOperationResult<int> DatExporter::ExtractAllFilesByType(const FILE_TYPE &type, std::string output_directory_path) {
        int success_exported = 0;

        auto operation = dat->getFileSystem().PerformOperationOnAllFiles([&success_exported, this, type, &output_directory_path](std::shared_ptr<SubFile>& file) -> void {
            if (file->FileType() != type || !dat->getFileSystem().CheckCorrectSubfile(file))
                return;
            SubfileData export_file = file->PrepareForExport(dat->getFileSystem().GetFileData(file->file_id(), 8));

            bool result;

            if (type == TEXT)
                result = WriteStringToFile(export_file.text_data, output_directory_path + std::to_string(file->file_id()) + file->Extension()).result;
            else
                result = export_file.binary_data.WriteToFile(output_directory_path + std::to_string(file->file_id()) + file->Extension());

            if (result)
                success_exported++;
        });
        return DatOperationResult<int>(success_exported, SUCCESS);
    }

    DatOperationResult<> DatExporter::ExtractFileById(long long file_id, std::string output_filename) {
        auto operation_GetFileData = dat->getFileSystem().GetFileData(file_id, 8);
        if (operation_GetFileData.result == ERROR)
            return DatOperationResult<>(ERROR, "EXTRACTFILEBYID: Can't get data for id = " + std::to_string(file_id));


        auto operation_GetFilePointer = dat->getFileSystem().GetFile(file_id);
        if (operation_GetFileData.result == ERROR)
            return DatOperationResult<>(ERROR, "EXTRACTFILEBYID: Can't get pointer for id = " + std::to_string(file_id));

        SubfileData export_data = operation_GetFilePointer.value->PrepareForExport(operation_GetFileData.value);

        if (export_data.Empty())
            return DatOperationResult<>(ERROR, "EXTRACTFILEBYID: Export data is empty for id = " + std::to_string(file_id));

        if (!export_data.binary_data.WriteToFile(output_filename + export_data.options["ext"].as<std::string>()))
            return DatOperationResult<>(ERROR, "EXTRACTFILEBYID: Cannot write to file" + output_filename);

        return DatOperationResult<>(SUCCESS);
    }

    DatOperationResult<int> DatExporter::ExtractAllFilesByType(const FILE_TYPE &type, Database *db) {
        if (!db)
            return DatOperationResult<int>(0, ERROR, "EXTRACTALLBYTYPETODB: database is nullptr");

        int success_exported = 0;
        auto operation = dat->getFileSystem().PerformOperationOnAllFiles([&success_exported, this, type, db](std::shared_ptr<SubFile>& file) {
            if (!dat->getFileSystem().CheckCorrectSubfile(file))
                return;

            SubfileData export_file = file->PrepareForExport(dat->getFileSystem().GetFileData(file->file_id(), 8));

            bool result = db->PushFile(export_file);

            if (result)
                success_exported++;
        });
        return DatOperationResult<int>(success_exported, SUCCESS);
    }

    DatOperationResult<> DatExporter::WriteStringToFile(const std::u16string &str, const std::string &path) {
        FILE* file = fopen(path.c_str(), "w");
        if (!file)
            return DatOperationResult<>(ERROR, "WRITESTRINGTOFILE: cant open file " + path);

        fprintf(file, "%s", (char *) str.c_str());
        fclose(file);
        return DatOperationResult<>();
    }

};